package com.smppw.modaq.application.components.report.parser.ai;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.smppw.modaq.application.components.ReportParseUtils;
import com.smppw.modaq.application.components.report.parser.ReportParserConstant;
import com.smppw.modaq.common.enums.ReportParseStatus;
import com.smppw.modaq.common.exception.ReportParseException;
import com.smppw.modaq.domain.dto.report.*;
import com.smppw.modaq.domain.mapper.EmailFieldMappingMapper;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;

@Component(ReportParserConstant.PARSER_AI_MONTHLY)
public class AIMonthlyReportParser extends AbstractAIReportParser<MonthlyReportData> {
    public AIMonthlyReportParser(EmailFieldMappingMapper fieldMappingMapper) {
        super(fieldMappingMapper);
    }

    @Override
    protected boolean isSupportAIParse() {
        return true;
    }

    @Override
    protected String prompt() {
        return "识别文件中的基金概况、净值信息表格数据，要求准确识别金额等小数的位数，去掉金额单位、英文和多余的空格，结果用json返回";
    }

    @Override
    protected void handleAiResult(String result) throws ReportParseException {
        try {
            JSONObject jsonObject = JSONUtil.parseObj(result);
            this.allInfoMap.putAll(jsonObject);
        } catch (Exception e) {
            throw new ReportParseException(ReportParseStatus.PARSE_HANDLE_FAIL);
        }
    }

    @Override
    @SuppressWarnings("unchecked")
    protected MonthlyReportData parseExtInfoAndSetData(ReportBaseInfoDTO reportInfo,
                                                       ReportFundInfoDTO fundInfo) throws ReportParseException {
        List<ReportNetReportDTO> dtos = ListUtil.list(true);
        for (Map.Entry<String, Object> entry : this.allInfoMap.entrySet()) {
            Object netInfo = entry.getValue();
            if (netInfo == null) {
                continue;
            }
            Map<String, Object> netInfoMap = (Map<String, Object>) netInfo;
            ReportNetReportDTO dto = this.buildDto(reportInfo.getFileId(), ReportNetReportDTO.class, netInfoMap);
            if (dto == null) {
                continue;
            }
            dto.setLevel(ReportParseUtils.matchFundLevel(entry.getKey()));
            dtos.add(dto);
        }
        MonthlyReportData reportData = new MonthlyReportData(reportInfo, fundInfo);
        reportData.setNetReport(dtos);
        return reportData;
    }

    @Override
    @SuppressWarnings("unchecked")
    protected ReportFundInfoDTO buildFundInfo(ReportParserParams params) {
        // 获取并移除基金概况信息
        Object fundInfo = this.allInfoMap.remove("基金概况");
        if (fundInfo == null) {
            throw new ReportParseException(ReportParseStatus.PARSE_FUND_INFO_FAIL, params.getFilename());
        }
        Map<String, Object> fundInfoMap = (Map<String, Object>) fundInfo;
        return this.buildDto(params.getFileId(), ReportFundInfoDTO.class, fundInfoMap);
    }
}
