package com.smppw.modaq.application.components.report.parser.ai;

import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.smppw.modaq.application.components.report.parser.ReportParserConstant;
import com.smppw.modaq.common.enums.ReportParseStatus;
import com.smppw.modaq.common.exception.ReportParseException;
import com.smppw.modaq.domain.dto.report.*;
import com.smppw.modaq.domain.mapper.EmailFieldMappingMapper;
import org.springframework.stereotype.Component;

/**
 * 其他格式的报告（只解析报告基本信息、基金基本信息）
 */
@Component(ReportParserConstant.PARSER_AI_OTHER)
public class AiOtherReportParser extends AbstractAIReportParser<ReportData> {
    public AiOtherReportParser(EmailFieldMappingMapper fieldMappingMapper) {
        super(fieldMappingMapper);
    }

    @Override
    protected String prompt() {
        return "识别文件中的基金名称、基金管理人、基金托管人和报告日期，如果无法识别就返回空字符，结果用json返回";
    }

    @Override
    protected boolean isSupportAIParse() {
        return true;
    }

    @Override
    protected void handleAiResult(String result) throws ReportParseException {
        try {
            JSONObject jsonObject = JSONUtil.parseObj(result);
            this.allInfoMap.putAll(jsonObject);
        } catch (Exception e) {
            throw new ReportParseException(ReportParseStatus.PARSE_HANDLE_FAIL);
        }
    }

    @Override
    protected ReportData parseExtInfoAndSetData(ReportBaseInfoDTO reportInfo,
                                                ReportFundInfoDTO fundInfo) throws ReportParseException {
        return new MonthlyReportData(reportInfo, fundInfo);
    }

    @Override
    protected ReportFundInfoDTO buildFundInfo(ReportParserParams params) {
        return this.buildDto(params.getFileId(), ReportFundInfoDTO.class, this.allInfoMap);
    }
}
