package com.smppw.modaq.infrastructure.util;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.StrUtil;
import net.sf.sevenzipjbinding.ExtractOperationResult;
import net.sf.sevenzipjbinding.IInArchive;
import net.sf.sevenzipjbinding.SevenZip;
import net.sf.sevenzipjbinding.SevenZipException;
import net.sf.sevenzipjbinding.impl.RandomAccessFileInStream;
import net.sf.sevenzipjbinding.simple.ISimpleInArchive;
import net.sf.sevenzipjbinding.simple.ISimpleInArchiveItem;
import org.apache.commons.compress.archivers.ArchiveEntry;
import org.apache.commons.compress.archivers.ArchiveException;
import org.apache.commons.compress.archivers.ArchiveInputStream;
import org.apache.commons.compress.archivers.ArchiveStreamFactory;
import org.apache.commons.compress.archivers.zip.ZipArchiveEntry;
import org.apache.commons.compress.archivers.zip.ZipFile;
import org.apache.commons.io.IOUtils;

import java.io.*;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Enumeration;
import java.util.List;

public class ExcelUtil {
    public static boolean isExcel(String fileName) {
        return StrUtil.isNotBlank(fileName) && (fileName.endsWith("xls") || fileName.endsWith("xlsx") || fileName.endsWith("XLS") || fileName.endsWith("XLSX"));
    }

    public static boolean isPdf(String fileName) {
        return StrUtil.isNotBlank(fileName) && (fileName.endsWith("pdf") || fileName.endsWith("PDF"));
    }

    public static boolean isZip(String fileName) {
        return StrUtil.isNotBlank(fileName) && (fileName.endsWith("zip") || fileName.endsWith("ZIP"));
    }

    public static boolean isHTML(String fileName) {
        return StrUtil.isNotBlank(fileName) && fileName.endsWith("html");
    }

    public static boolean isRAR(String fileName) {
        return StrUtil.isNotBlank(fileName) && (fileName.endsWith("rar") || fileName.endsWith("RAR"));
    }

    public static List<String> extractCompressedFiles(String zipFilePath, String destFilePath) throws IOException, ArchiveException {
        List<String> filePathList = CollUtil.newArrayList();

        File destFile = FileUtil.file(destFilePath);
        if (!destFile.exists()) {
            Files.createDirectories(destFile.toPath());
        }

        try (BufferedInputStream fis = new BufferedInputStream(new FileInputStream(zipFilePath));
             ArchiveInputStream<? extends ArchiveEntry> ais = new ArchiveStreamFactory().createArchiveInputStream(fis)) {
            ArchiveEntry entry;
            int i = 1;
            while ((entry = ais.getNextEntry()) != null) {
                String name = entry.getName();
                if (name.startsWith("__MACOSX/")) {
                    continue;
                }
                String zipFilename = FileUtil.getName(destFilePath);
                if (name.contains("确认")) {
                    zipFilename += "_确认单";
                }
                String ext = FileUtil.extName(name);
                name = zipFilename + "_" + i + "." + ext;
                File entryFile = FileUtil.file(destFilePath, name);
                i++;
                if (entry.isDirectory()) {
                    Files.createDirectories(entryFile.toPath());
                } else {
                    try (FileOutputStream fos = new FileOutputStream(entryFile)) {
                        IOUtils.copy(ais, fos);
                        filePathList.add(entryFile.getPath());
                    }
                }
            }
        } catch (Exception e) {
            if (e.getMessage() != null
                    && (e.getMessage().contains("split")
                    || e.getMessage().contains("volume"))) {
                filePathList.addAll(extractSplitZip(zipFilePath, destFilePath));
            } else {
                throw e;
            }
        }

        return filePathList;
    }

//    public static List<String> extractRar(String inputFilePath, String outputDirPath) throws IOException {
//        List<String> fileList = new ArrayList<>();
//        // 创建Archive对象，用于读取rar压缩文件格式
//        try {
//            String zipFilename = FileUtil.getName(inputFilePath);
//            Archive archive = new Archive(new FileInputStream(inputFilePath));
//            // 读取压缩文件中的所有子目录或子文件（FileHeader对象）
//            List<FileHeader> fileHeaderList = archive.getFileHeaders();
//            int i = 1;
//            // 遍历子目录和子文件
//            for (FileHeader fd : fileHeaderList) {
//                String fileName = fd.getFileName();
//                String ext = FileUtil.extName(fileName);
//                fileName = zipFilename + "_" + i + "." + ext;
//                i++;
//                File f = FileUtil.file(outputDirPath + File.separator + fileName);
//                if (fd.isDirectory()) {
//                    // 创建新子目录
//                    Files.createDirectories(f.toPath());
//                } else {
//                    // 创建新子文件
//                    Files.createFile(f.toPath());
//                    // 获取压缩包中的子文件输出流
//                    InputStream in = archive.getInputStream(fd);
//                    // 复制文件输入流至新子文件
//                    FileUtil.copyFile(in, f);
//                    fileList.add(f.getAbsolutePath());
//                }
//            }
//        } catch (RarException e) {
//            try {
//                List<File> extract = Junrar.extract(new File(inputFilePath), new File(outputDirPath));
//                for (File file : extract) {
//                    fileList.add(file.getAbsolutePath());
//                }
//            } catch (RarException ex) {
//                throw new RuntimeException(ex);
//            }
//        }
//        return fileList;
//    }

    public static List<String> extractSplitZip(String zipFilePath, String destFilePath) throws IOException {
        List<String> resultList = ListUtil.list(false);
        File file = new File(zipFilePath);
        try (ZipFile zipFile = ZipFile.builder().setFile(file).get()) {
            Enumeration<ZipArchiveEntry> entries = zipFile.getEntries();
            while (entries.hasMoreElements()) {
                ZipArchiveEntry entry = entries.nextElement();
                // 解压到目标目录
                try (InputStream is = zipFile.getInputStream(entry)) {
                    Path path = Paths.get(destFilePath, entry.getName());
                    FileUtil.del(path);
                    Files.copy(is, path);
                    resultList.add(path.toAbsolutePath().toString());
                }
            }
        }
        return resultList;
    }

    public static List<String> extractRar5(String rarFilePath, String outputDir) throws Exception {
        // 初始化 SevenZipJBinding 本地库
        SevenZip.initSevenZipFromPlatformJAR();

        RandomAccessFile randomAccessFile = null;
        IInArchive inArchive = null;

        List<String> resultList = ListUtil.list(false);
        try {
            // 打开 RAR 文件
            randomAccessFile = new RandomAccessFile(rarFilePath, "r");
            inArchive = SevenZip.openInArchive(null, new RandomAccessFileInStream(randomAccessFile));

            // 获取压缩包中的文件列表
            ISimpleInArchive simpleInArchive = inArchive.getSimpleInterface();
            for (ISimpleInArchiveItem item : simpleInArchive.getArchiveItems()) {
                if (!item.isFolder()) {
                    resultList.add(extractItem(item, outputDir));
                }
            }
        } finally {
            // 释放资源
            if (inArchive != null) {
                inArchive.close();
            }
            if (randomAccessFile != null) {
                randomAccessFile.close();
            }
        }
        return resultList;
    }

    private static String extractItem(ISimpleInArchiveItem item, String outputDir) throws SevenZipException {
        String filePath = outputDir + File.separator + item.getPath();
        File outputFile = FileUtil.file(filePath);

        // 创建父目录
        File parentDir = outputFile.getParentFile();
        if (!parentDir.exists() && !parentDir.mkdirs()) {
            throw new SevenZipException("无法创建目录: " + parentDir.getAbsolutePath());
        }

        // 提取文件内容
        try (FileOutputStream fos = new FileOutputStream(outputFile)) {
            ExtractOperationResult result = item.extractSlow(data -> {
                try {
                    fos.write(data);
                    return data.length; // 返回写入的字节数
                } catch (IOException e) {
                    throw new SevenZipException("写入文件失败", e);
                }
            });

            if (result != ExtractOperationResult.OK) {
                throw new SevenZipException("解压失败: " + result);
            }
        } catch (IOException e) {
            throw new SevenZipException("文件操作失败", e);
        }
        return outputFile.getAbsolutePath();
    }

    public static void main(String[] args) throws Exception {
        String zipFilePath = "D:\\home\\wwwroot\\mo_report_file\\wangzaijun@simuwang.com\\20250321\\20250321143709排排网确认单.rar";
        String destFilePath = "D:\\home\\wwwroot\\mo_report_file\\wangzaijun@simuwang.com\\20250321";
        List<String> strings = extractRar5(zipFilePath, destFilePath);
        for (String string : strings) {

        }
//        List<String> fileList = extractCompressedFiles(zipFilePath, destFilePath);
//        for (String s : fileList) {
//            System.out.println(s);
//        }
    }
}
