package com.smppw.modaq.infrastructure.util;

import cn.hutool.core.exceptions.ExceptionUtil;
import jakarta.mail.Part;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.*;
import java.nio.file.Files;

public class FileUtil {

    private static final Logger logger = LoggerFactory.getLogger(FileUtil.class);

    public static void writeFile(String fileName, String content) {
        writeFile(new File(fileName), content, "UTF-8");
    }

    public static void writeFile(File file, String content, String charsets) {
        writeFile(file, content, charsets, false);
    }

    public static void writeFile(File file, String content, String charsets, boolean append) {
        Writer fw = null;
        String fileName = file.getAbsolutePath();

        try {
            File folder = file.getParentFile();
            if (!folder.exists()) {
                folder.setExecutable(true, false);
                folder.setReadable(true, false);
                folder.mkdirs();
            }

            if (append) {
                fw = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(file, append), charsets));
            } else {
                fw = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(file), charsets));
            }

            fw.write(content);
            fw.flush();
        } catch (IOException var15) {
            logger.error("写入失败！file=" + fileName);
            logger.error("堆栈信息:{}", ExceptionUtil.stacktraceToString(var15));
        } finally {
            try {
                if (fw != null) {
                    fw.close();
                }
            } catch (IOException var14) {
                logger.error("写入的IO关闭失败！file= " + fileName);
                logger.error("堆栈信息:{}", ExceptionUtil.stacktraceToString(var14));
            }

        }
    }

    public static void saveFile(File saveFile, Part part) throws Exception {
        try (InputStream is = part.getInputStream()) {
            Files.copy(is, saveFile.toPath());
        }
    }
}
